Package: NPARC
Type: Package
Title: Non-parametric analysis of response curves for thermal proteome
        profiling experiments
Version: 1.22.0
Author: 
    Dorothee Childs,
    Nils Kurzawa
Maintainer: Nils Kurzawa <nilskurzawa@gmail.com>
Description: Perform non-parametric analysis of response curves as described by Childs, Bach, Franken et al. (2019): Non-parametric analysis of thermal proteome profiles reveals novel drug-binding proteins.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.1.1
Depends: R (>= 4.0.0)
Imports: dplyr, tidyr, BiocParallel, broom, MASS, rlang, magrittr,
        stats, methods
Suggests: testthat, devtools, knitr, rprojroot, rmarkdown, ggplot2,
        BiocStyle
VignetteBuilder: knitr
biocViews: Software, Proteomics
git_url: https://git.bioconductor.org/packages/NPARC
git_branch: RELEASE_3_22
git_last_commit: 92e6972
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:29:58 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(NPARCfit)
export(NPARCtest)
export(fitSingleSigmoid)
export(getParams)
export(runNPARC)
import(BiocParallel)
import(dplyr)
import(tidyr)
importFrom(MASS,fitdistr)
importFrom(broom,augment)
importFrom(magrittr,"%<>%")
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,D)
importFrom(stats,coefficients)
importFrom(stats,fitted)
importFrom(stats,integrate)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nls)
importFrom(stats,nls.control)
importFrom(stats,nobs)
importFrom(stats,p.adjust)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,resid)
importFrom(stats,runif)
importFrom(stats,vcov)

# NPARC
Nonparametric Analysis of Response Curves from Thermal Proteome Profiling Experiments.    
R package implementation of the method described by [Childs, Bach, Franken et al. (2019) Mol. Cell. Proteomics](https://doi.org/10.1074/mcp.TIR119.001481).

## Installation

### Installation from Bioconductor (recommended)

`NPARC` is in the process of being made available through Bioconductor! Therefore, the most reliable way to install it is via BiocManager:

```{R}
if (!requireNamespace("BiocManager", quietly = TRUE))
install.packages("BiocManager")
BiocManager::install("NPARC")
```

### Installation of the development version

```{R}
if (!requireNamespace("BiocManager", quietly = TRUE))
install.packages("BiocManager")
BiocManager::install(“Huber-group-EMBL/NPARC”)
```

## Getting started

The easiest way of learning how to use the `NPARC` package is to browse it's vignette:
```{R}
library(NPARC)
browseVignettes("NPARC")
```

evalModels <- function(fits, groups, x){
  # Invoke model evaluation

  modelMetrics <- assessModelMetrics(fits = fits, x = x, groups = groups)
  modelPredictions <- augmentModels(fits = fits, groups = groups)

  return(list(modelMetrics = modelMetrics, modelPredictions = modelPredictions))
}

augmentModels <- function(fits, groups){
  # Augment model to obtain data frame with measurements, predictions and residuals.

  # ---- Computing model predictions and residuals ----
  message("Computing model predictions and residuals ...")

  modelPredictions <- fits %>%
    filter(.data$successfulFit) %>%
    group_by_at(c("iter", "id", groups)) %>%
    do(augment(.data$fittedModel[[1]])) %>%
    ungroup %>%
    right_join(fits %>% select(!!c("iter", "id", groups)), by = c("iter", "id", groups)) %>% # fill for unsuccessful fits
    select(-.data$iter)

  message("... complete.\n")

  return(modelPredictions)

}

assessModelMetrics <- function(fits, x, groups){
  # Invoke calculation of performance metrics per model

  message("Evaluating models ...")

  metrics <- fits %>%
    group_by_at(c("iter", "id", groups)) %>%
    do(assessSingleModel(nls_obj = .data$fittedModel[[1]],
                         xVec = unique(x))) %>%
    ungroup %>%
    select(-.data$iter)

  message("... complete.\n")

  return(metrics)
}

#' @importFrom methods is
assessSingleModel <- function(nls_obj, xVec){

  if (any(is.na(xVec))){
    stop("Temperature vector contains missing values. Cannot integrate over these values to compute the curve area.")
  }

  tm = a = b = pl = aumc = tm_sd = rss = logL = nCoeffs = nFitted = resid_sd <- NA
  conv <- FALSE
  if (!is(nls_obj, "try-error")){
    #meltCurve <- Reduce(paste, deparse(formula(nls_obj))) %>% gsub("y ~ ", "", .)
    pars <- coefficients(nls_obj)
    nCoeffs <- length(pars)
    a <- pars[["a"]]
    b <- pars[["b"]]
    pl <- pars[["Pl"]]
    yVec <- fitted(nls_obj) + resid(nls_obj)
    tm_fct <- parse(text = "a / (b - log((1-pl)/(1/2 - pl) - 1))")
    suppressWarnings(tm <- eval(tm_fct))
    dtm_da <-D(tm_fct, "a")
    dtm_db <-D(tm_fct, "b")
    dtm_dpl <-D(tm_fct, "pl")
    suppressWarnings(grad_tm <- c(a = eval(dtm_da), b = eval(dtm_db), pl = eval(dtm_dpl))) # derivatives of 'tm' .w.r.t. 'a' and 'b'
    covmat <- try(vcov(nls_obj), silent = TRUE) # var-covar matrix of estimators 'a' and 'b'
    if (!inherits(covmat, "try-error")){
      var_tm <- grad_tm %*% covmat[c("a", "b", "Pl"), c("a", "b", "Pl")] %*% grad_tm
      tm_sd <- as.numeric(sqrt(var_tm)) # squash matrix to vector
    }
    ## Compute area under the melting curve:
    int <- try(integrate(function(x,m) predict(m, newdata = list(x=x)),
                         min(xVec), max(xVec), nls_obj), silent = TRUE)
    if (!inherits(int, "try-error")){
      aumc <- int$value
    }

    nFitted <- nobs(nls_obj)
    rss <- sum(resid(nls_obj)^2, na.rm = TRUE)
    resid_sd  <- sqrt(rss/nFitted)
    logL <- -nFitted/2 * log(2*pi*resid_sd^2) - rss/(2*resid_sd^2) #loglik <- logLik(m)
    nonNAs <- sum(!is.na(resid(nls_obj)))

    conv <- nls_obj$convInfo$isConv
  }
  return(data.frame(tm = tm, a = a,  b = b,  pl = pl, aumc = aumc,
                    resid_sd = resid_sd, rss = rss, loglik = logL,
                    tm_sd = tm_sd, nCoeffs = nCoeffs, nFitted = nFitted, conv = conv))
}

#' TPP dataset of staurosporine treated cells.
#'
#' Data from a thermal proteome profiling (TPP) experiment investigating the
#' ATP-competitive pan-kinase inhibitor staurosporine on K562 cells.
#' The data has been downloaded the data from the supplement of the respective publication and converted into tidy format.
#'
#' @docType data
#'
#' @usage data(stauro_TPP_data_tidy)
#'
#' @format An object of class \code{"data.frame"}
#'
#' @keywords datasets
#'
#' @references  Savitski et al. (2014): Tracking cancer drugs in living cells by thermal profiling of the proteome. Science 346, 1255784.
"stauro_TPP_data_tidy"

#' Fit null and alternative models for Non-parametric analysis of response curves
#'
#' Fit melting curve and return model metrics as well as predictions for the null and alternative models.
#'
#' @param x numeric vector of the independent variables (typically temperature)
#' @param y numeric vector of the dependent variables (typically relative abundance measurements)
#' @param id character vector with the protein ID to which each each data point belongs.
#' @param control list of parameters used to control specific parts of the analyse
#' @param BPPARAM BiocParallel parameter object to invoke curve fitting in parallel. Default: BiocParallel::SerialParam()
#' @param returnModels boolean value. If true, the fitted models are returned together with the test results
#' @param groupsNull one or more vectors with grouping variables for the null models. See details.
#' @param groupsAlt one or more vectors with grouping variables for the alternative models. See details.
#' @return data frame with fitted model parameters and additional columns listing e.g. residuals sum of squares 
#' @details
#' \code{groupsNull} or \code{groupsAlt} can either be a single vector each, or data.frames of the same length as \code{x} and \code{y} with one column per factor
#'
#' @export
#' @examples
#' data(stauro_TPP_data_tidy)
#' df <- dplyr::filter(stauro_TPP_data_tidy, grepl("CDK|GTP|CRK", uniqueID))
#' testResults <- NPARCfit(x = df$temperature,
#'                      y = df$relAbundance,
#'                      id = df$uniqueID,
#'                      groupsAlt = df$compoundConcentration)
#'
#'
#' @export
NPARCfit <- function(x, y,
                     id,
                     control = getParams(),
                     groupsNull = NULL,
                     groupsAlt,
                     BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
                     returnModels = FALSE){

  fitResNull <- invokeParallelFits(x = x,
                                   y = y,
                                   id = id,
                                   groups = groupsNull,
                                   BPPARAM = BPPARAM,
                                   maxAttempts = control$maxAttempts,
                                   returnModels = returnModels,
                                   start = control$start)

  fitResAlt <-  invokeParallelFits(x = x,
                                   y = y,
                                   id = id,
                                   groups = groupsAlt,
                                   BPPARAM = BPPARAM,
                                   maxAttempts = control$maxAttempts,
                                   returnModels = returnModels,
                                   start = control$start)

  predictions <- bind_rows(null = fitResNull$modelPredictions,
                           alternative = fitResAlt$modelPredictions,
                           .id = "modelType")

  metrics <- bind_rows(null = fitResNull$modelMetrics,
                       alternative = fitResAlt$modelMetrics,
                       .id = "modelType")

  return(list(predictions = predictions,
              metrics = metrics))

}


invokeParallelFits <- function(x, y,
                               id,
                               groups,
                               BPPARAM,
                               seed,
                               maxAttempts,
                               returnModels,
                               start){

  if (is.null(groups)){
    groups <- as.data.frame(matrix(nrow = length(id), ncol = 0))
  } else if (is.vector(groups)){
    groups <- data.frame(group = groups, stringsAsFactors = FALSE)
  }
  group_names <- colnames(groups)

  groups <- groups %>%
    as_tibble() %>% 
    mutate(id = id) %>%
    unite("iter", !!c("id", group_names), remove = FALSE)

  # ---- Fit models ----
  t1 <- Sys.time()
  message("Starting model fitting...")

  models <- fitAllModels(x = x,
                         y = y,
                         iter = groups$iter,
                         BPPARAM = BPPARAM,
                         maxAttempts = maxAttempts,
                         start = start)


  message("... complete")
  timeDiff <- Sys.time() - t1
  message("Elapsed time: ", round(timeDiff, 2), " ", units(timeDiff), "\n")

  # ---- Flag successful model fits ----
  message("Flagging successful model fits...")

  fits <- groups %>%
    group_by_at(c("iter", "id", group_names)) %>%
    distinct_at(c("iter", "id", group_names)) %>%
    do(fittedModel = models[[.data$iter]]) %>%
    ungroup %>%
    ## Mark proteins where model fit was not successful
    mutate(successfulFit = 
               (vapply(.data$fittedModel, class, FUN.VALUE = "nls") != 
                    "try-error"))

  message("... complete.\n")

  # ---- Evaluate successful model fits ----
  message("Evaluating model fits...")

  fitResults <- evalModels(fits = fits,
                           x = x,
                           groups = group_names)

  if (returnModels) {
    fitResults$fittedModels <- fits
  } else {
    fitResults$fittedModels <- NULL
  }

  return(fitResults)
}

fitAllModels <- function(x,
                         y,
                         iter,
                         BPPARAM,
                         seed,
                         maxAttempts,
                         start){

  # ---- Fit sigmoid models ----
  models <- BiocParallel::bplapply(unique(iter),
                                   function(i) fitToSubset(subset = i,
                                                           x = x,
                                                           y = y,
                                                           iter = iter,
                                                           maxAttempts = maxAttempts,
                                                           start = start),
                                   BPPARAM = BPPARAM)
  gc()

  # ---- Return model fits ----
  model_names <- vapply(models, function(m) attributes(m)$iter, 
                        FUN.VALUE = "geneName_ipiId")

  names(models) <- model_names

  #   message("\nFor ", sum(allRSS$repeats == 0) , " proteins, the models successfully converged and produced nonnegative RSS-differences in the first iteration.",
  #           "\nFor ", sum(allRSS$repeats > 0 & allRSS$repeats < 10), " proteins, nonnegative RSS-differences could be obtained after repeating the fit with different start parameters.")


  return(models)
}

fitToSubset <- function(subset, x, y, iter, seed, maxAttempts, start){

  idx <- which(iter == subset)

  model <- repeatSingleFit(x = x[idx],
                           y = y[idx],
                           maxAttempts = maxAttempts,
                           start = start)

  attr(model, "iter") <- subset

  return(model)

}


repeatSingleFit <- function(x, y,
                            start,
                            maxAttempts = 100){
  # Wrapper to repeat the fit until model has converged

  i <- 0
  doFit <- TRUE
  doVaryPars <- FALSE

  while (doFit){
    startTmp <- start * (1 + doVaryPars*runif(1, -0.5, 0.5))
    m <- fitSingleSigmoid(x = x, y = y, start = startTmp)
    i <- i + 1
    doFit <- inherits(m, "try-error") & i < maxAttempts
    doVaryPars <- TRUE
  }

  return(m)
}

#' Fit sigmoid model
#'
#' @param x numeric vector of the independent variables (typically temperature)
#' @param y numeric vector of the dependent variables (typically relative abundance measurements)
#' @param start numeric vector of start parameters for the melting curve equation
#' @return model summary of type "nls"
#' @export
#' @details
#' Fits the following function to the data:
#' \eqn{y = (1 - Pl)  / (1+exp((b - a/x))) + Pl}
#' @examples
#' data(stauro_TPP_data_tidy)
#' stk4 <- dplyr::filter(stauro_TPP_data_tidy, grepl("STK4", uniqueID))
#' fitSingleSigmoid(stk4$temperature, stk4$relAbundance)
fitSingleSigmoid <- function(x, y, start=c(Pl = 0, a = 550, b = 10)){
  try(nls(formula = y ~ (1 - Pl)  / (1+exp((b - a/x))) + Pl,
          start = start,
          data = list(x=x, y=y),
          na.action = na.omit,
          algorithm = "port",
          lower = c(0.0, 1e-5, 1e-5),
          upper = c(1.5, 15000, 250),
          control = nls.control(maxiter=50)),
      silent = TRUE)
}

#' Control parameters for model fitting
#'
#' @param maxAttempts Number of resampling steps in case of unsuccessful model fits
#' @param start Numeric vector of start parameters for the melting curve equation
#' @return list of two elements: 1) "start" listing the starting parameters for
#'  melting curve fitting, 2) "maxAttempts" listing the maximal number of attempts
#'  the fit should be allowed
#' @examples
#' data(stauro_TPP_data_tidy)
#' df <- dplyr::filter(stauro_TPP_data_tidy, grepl("MAPK|ATP|CDK|GTP|CRK", uniqueID))
#' testResults <- runNPARC(x = df$temperature,
#'                      y = df$relAbundance,
#'                      id = df$uniqueID,
#'                      groupsAlt = df$compoundConcentration,
#'                      dfType = "empirical",
#'                      control = getParams(maxAttempts = 50))
#'
#' @export
#'
getParams <- function(start = c(Pl = 0, a = 550, b = 10),
                      maxAttempts = 100){

  list(start = start,
       maxAttempts = maxAttempts)
}

source(file.path(rprojroot::find_package_root_file(), "tests/init_tests.R"))

context("invokeParallelFits_allok_rss0")
test_that("invokeParallelFits_allok_rss0", {

  result <- invokeParallelFits(x = ATP_targets_stauro$temperature,
                               y = ATP_targets_stauro$relAbundance,
                               id = ATP_targets_stauro$uniqueID,
                               groups = ATP_targets_stauro$uniqueID,
                               BPPARAM = BiocParallel::SerialParam(),
                               maxAttempts = 100,
                               returnModels = TRUE,
                               start = c(Pl = 0, a = 550, b = 10))

  rss0_new <- result$modelMetrics$rss

  expect_equal(unname(rss0_new)[-16], rss0_ref[-16]) # position 16: ATP5G1_IPI00009075 -> was a different seed used to resample due to negative RSS-Diff?
})


context("invokeParallelFits_allok_rss1")
test_that("invokeParallelFits_allok_rss1", {

  result <- invokeParallelFits(x = ATP_targets_stauro$temperature,
                               y = ATP_targets_stauro$relAbundance,
                               id = ATP_targets_stauro$uniqueID,
                               groups = ATP_targets_stauro$compoundConcentration,
                               BPPARAM = BiocParallel::SerialParam(),
                               maxAttempts = 100,
                               returnModels = TRUE,
                               start = c(Pl = 0, a = 550, b = 10))

  rss1_new <- result$modelMetrics %>%
    group_by(id) %>%
    summarise(rss = sum(rss))


  expect_equal(rss1_new$rss, rss1_ref)
})

context("fitAllModels_allok_rss0")
test_that("fitAllModels_allok_rss0", {

  models <- fitAllModels(x = ATP_targets_stauro$temperature,
                         y = ATP_targets_stauro$relAbundance,
                         iter = ATP_targets_stauro$uniqueID,
                         BPPARAM = BiocParallel::SerialParam(),
                         maxAttempts = 100,
                         start = c(Pl = 0, a = 550, b = 10))

  rss0_new <- sapply(models, function(m) {
    ifelse(inherits(m , "try-error"), NA, m$m$deviance())
  })

  expect_equal(unname(rss0_new)[-16], rss0_ref[-16]) # position 16: ATP5G1_IPI00009075 -> was a different seed used to resample due to negative RSS-Diff?
})


context("fitAllModels_allok_rss1")
test_that("fitAllModels_allok_rss1", {

  models <- fitAllModels(x = ATP_targets_stauro$temperature,
                         y = ATP_targets_stauro$relAbundance,
                         iter = paste(ATP_targets_stauro$uniqueID, ATP_targets_stauro$compoundConcentration),
                         BPPARAM = BiocParallel::SerialParam(),
                         maxAttempts = 100,
                         start = c(Pl = 0, a = 550, b = 10))

  rss1_new <- sapply(models, function(m) {
    ifelse(inherits(m , "try-error"), NA, m$m$deviance())
  })

  rss1_new <- tibble(groups = names(rss1_new), rss1 = rss1_new) %>%
    separate("groups", c("id", "compoundConcentration"), remove = FALSE, sep = " ") %>%
    group_by(id) %>%
    summarise(rss1 = sum(rss1))

  expect_equal(unname(rss1_new$rss1), rss1_ref)
})

#' Non-parametric analysis of response curves
#'
#' Wrapper function for melting curve fitting and hypothesis testing.
#'
#' @param x numeric vector of the independent variables (typically temperature)
#' @param y numeric vector of the dependent variables (typically relative abundance measurements)
#' @param id character vector with the protein ID to which each each data point belongs.
#' @param control list of parameters used to control specific parts of the analyse
#' @param BPPARAM BiocParallel parameter object to invoke curve fitting in parallel. Default: BiocParallel::SerialParam()
#' @param dfType character value indicating the method for degrees of freedom computation for the F-test. 
#'  Theoretical yields the text-book solution. Empirical yields estimates derived from the distribution moments of the RSS.
#' @param groupsNull one or more vectors with grouping variables for the null models. See details.
#' @param groupsAlt one or more vectors with grouping variables for the alternative models. See details.
#' @return data frame with fitted model parameters and additional columns listing e.g. residuals sum of squares of
#'  null and alterantive model
#' @details
#' \code{groupsNull} or \code{groupsAlt} can either be a single vector each, or data.frames of the same length as \code{x} and \code{y} with one column per factor
#'
#' @export
#'
#' @examples
#' data(stauro_TPP_data_tidy)
#' df <- dplyr::filter(stauro_TPP_data_tidy, grepl("CDK|GTP|CRK", uniqueID))
#' testResults <- runNPARC(x = df$temperature,
#'                      y = df$relAbundance,
#'                      id = df$uniqueID,
#'                      groupsAlt = df$compoundConcentration,
#'                      dfType = "empirical")
runNPARC <- function(x,
                     y,
                     id,
                     groupsNull = NULL,
                     groupsAlt,
                     BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
                     dfType = c("theoretical", "empirical"),
                     control = getParams()){

  fits <- NPARCfit(x = x,
                   y = y,
                   id = id,
                   groupsNull = groupsNull,
                   groupsAlt = groupsAlt,
                   BPPARAM = BPPARAM,
                   returnModels = FALSE,
                   control = control)

    modelMetrics <- fits$metrics

    testRes <-  NPARCtest(modelMetrics, dfType = dfType)

  return(testRes)
}

source(file.path(rprojroot::find_package_root_file(), "tests/init_tests.R"))

test_that("invokeNPARC_allok", {
  results <- runNPARC(x = ATP_targets_stauro$temperature,
                      y = ATP_targets_stauro$relAbundance,
                      id = ATP_targets_stauro$uniqueID,
                      groupsAlt = ATP_targets_stauro$compoundConcentration,
                      dfType = "theoretical")

  expect_equal(results$pAdj, pAdj_ref, tolerance = 1e-6)

})

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----dependencies, message=FALSE----------------------------------------------
library(dplyr)
library(magrittr)
library(ggplot2)
library(broom)
library(knitr)
library(NPARC)

## ----load_data----------------------------------------------------------------
data("stauro_TPP_data_tidy")

## -----------------------------------------------------------------------------
df <- stauro_TPP_data_tidy

## ----data_head----------------------------------------------------------------
df %>% 
  mutate(compoundConcentration = factor(compoundConcentration), 
         replicate = factor(replicate), 
         dataset = factor(dataset)) %>% 
  summary()


## ----filter_qupm_or_NAs, results='asis'---------------------------------------
df %<>% filter(uniquePeptideMatches >= 1)
df %<>% filter(!is.na(relAbundance))

## ----count_curves_per_protein-------------------------------------------------
# Count full curves per protein
df %<>%
  group_by(dataset, uniqueID) %>%
  mutate(n = n()) %>%
  group_by(dataset) %>%
  mutate(max_n = max(n)) %>% 
  ungroup

table(distinct(df, uniqueID, n)$n)

## -----------------------------------------------------------------------------
# Filter for full curves per protein:
df %<>% 
  filter(n == max_n) %>%
  dplyr::select(-n, -max_n)

## ----select_stk4--------------------------------------------------------------
stk4 <- filter(df, uniqueID == "STK4_IPI00011488")

## -----------------------------------------------------------------------------
stk4 %>% filter(compoundConcentration == 20, replicate == 1) %>% 
  dplyr::select(-dataset) %>% kable(digits = 2)

## ----plot_stk4----------------------------------------------------------------
stk4_plot_orig <- ggplot(stk4, aes(x = temperature, y = relAbundance)) +
  geom_point(aes(shape = factor(replicate), color = factor(compoundConcentration)), size = 2) +
  theme_bw() +
  ggtitle("STK4") +
  scale_color_manual("staurosporine (mu M)", values = c("#808080", "#da7f2d")) +
  scale_shape_manual("replicate", values = c(19, 17))

print(stk4_plot_orig)

## ----fit_null_stk4------------------------------------------------------------
nullFit <- NPARC:::fitSingleSigmoid(x = stk4$temperature, y = stk4$relAbundance)

## ----summarize_null_stk4------------------------------------------------------
summary(nullFit)

## ----augment_null_stk4--------------------------------------------------------
nullPredictions <- broom::augment(nullFit)

## ----head_augment_result------------------------------------------------------
nullPredictions %>% filter(x %in% c(46, 49)) %>% kable()

## ----add_null_resids_stk4-----------------------------------------------------
stk4$nullPrediction <- nullPredictions$.fitted
stk4$nullResiduals <- nullPredictions$.resid

stk4_plot <- stk4_plot_orig + geom_line(data = stk4, aes(y = nullPrediction))

print(stk4_plot)

## ----fit_alternative_stk4-----------------------------------------------------
alternativePredictions <- stk4 %>%
# Fit separate curves per treatment group:
  group_by(compoundConcentration) %>%
  do({
    fit = NPARC:::fitSingleSigmoid(x = .$temperature, y = .$relAbundance, start=c(Pl = 0, a = 550, b = 10))
    broom::augment(fit)
  }) %>%
  ungroup %>%
  # Rename columns for merge to data frame:
  dplyr::rename(alternativePrediction = .fitted,
                alternativeResiduals = .resid,
                temperature = x,
                relAbundance = y)

## ----add_alternative_resids_stk4----------------------------------------------
stk4 <- stk4 %>%
  left_join(alternativePredictions, 
            by = c("relAbundance", "temperature", 
                   "compoundConcentration")) %>%
  distinct()

## ----plot_Fig_2_A_B-----------------------------------------------------------
stk4_plot <- stk4_plot +
  geom_line(data = distinct(stk4, temperature, compoundConcentration, alternativePrediction), 
            aes(y = alternativePrediction, color = factor(compoundConcentration)))

print(stk4_plot)

## ----compute_rss_stk4---------------------------------------------------------
rssPerModel <- stk4 %>%
  summarise(rssNull = sum(nullResiduals^2),
            rssAlternative = sum(alternativeResiduals^2))

kable(rssPerModel, digits = 4)

## -----------------------------------------------------------------------------
BPPARAM <- BiocParallel::SerialParam(progressbar = FALSE)

## ----fit_all_proteins---------------------------------------------------------
fits <- NPARCfit(x = df$temperature, 
                 y = df$relAbundance, 
                 id = df$uniqueID, 
                 groupsNull = NULL, 
                 groupsAlt = df$compoundConcentration, 
                 BPPARAM = BPPARAM,
                 returnModels = FALSE)

str(fits, 1)

## ----show_fit_results---------------------------------------------------------
fits$metrics %>% 
  mutate(modelType = factor(modelType), nCoeffs = factor(nCoeffs), nFitted = factor(nFitted), group = factor((group))) %>% 
  summary

fits$predictions %>% 
  mutate(modelType = factor(modelType), group = factor((group))) %>% 
  summary

## ----show_stk4_results--------------------------------------------------------
stk4Metrics <- filter(fits$metrics, id == "STK4_IPI00011488")

rssNull <- filter(stk4Metrics, modelType == "null")$rss
rssAlt <- sum(filter(stk4Metrics, modelType == "alternative")$rss) # Summarize over both experimental groups

rssNull
rssAlt

## ----plot_stk4_results--------------------------------------------------------
stk4Predictions <- filter(fits$predictions, modelType == "alternative", id == "STK4_IPI00011488")

stk4_plot_orig +
  geom_line(data = filter(stk4Predictions, modelType == "alternative"), 
            aes(x = x, y = .fitted, color = factor(group))) +
    geom_line(data = filter(stk4Predictions, modelType == "null"), 
            aes(x = x, y = .fitted))

## ----testDOFiid---------------------------------------------------------------
modelMetrics <- fits$metrics 
fStats <- NPARCtest(modelMetrics, dfType = "theoretical")

## ----plotF_DOFiid-------------------------------------------------------------
fStats %>% 
  filter(!is.na(pAdj)) %>%
  distinct(nFittedNull, nFittedAlt, nCoeffsNull, nCoeffsAlt, df1, df2) %>%
  kable()

## ----plotFiid-----------------------------------------------------------------
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_density(aes(x = fStat), fill = "steelblue", alpha = 0.5) +
  geom_line(aes(x = fStat, y = df(fStat, df1 = df1, df2 = df2)), color = "darkred", size = 1.5) +
  theme_bw() +
  # Zoom in to small values to increase resolution for the proteins under H0:
  xlim(c(0, 10))

## ----plotPiid-----------------------------------------------------------------
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_histogram(aes(x = pVal, y = ..density..), fill = "steelblue", alpha = 0.5, boundary = 0, bins = 30) +
  geom_line(aes(x = pVal, y = dunif(pVal)), color = "darkred", size = 1.5) +
  theme_bw()

## ----testDOFemp---------------------------------------------------------------
modelMetrics <- fits$metrics 
fStats <- NPARCtest(modelMetrics, dfType = "empirical")

## ----plotFnew-----------------------------------------------------------------
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_density(aes(x = fStat), fill = "steelblue", alpha = 0.5) +
  geom_line(aes(x = fStat, y = df(fStat, df1 = df1, df2 = df2)), color = "darkred", size = 1.5) +
  theme_bw() +
  # Zoom in to small values to increase resolution for the proteins under H0:
  xlim(c(0, 10))

## ----plotPnew-----------------------------------------------------------------
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_histogram(aes(x = pVal, y = ..density..), fill = "steelblue", alpha = 0.5, boundary = 0, bins = 30) +
  geom_line(aes(x = pVal, y = dunif(pVal)), color = "darkred", size = 1.5) +
  theme_bw()

## ----selectHits---------------------------------------------------------------
topHits <- fStats %>% 
  filter(pAdj <= 0.01) %>%
  dplyr::select(id, fStat, pVal, pAdj) %>%
  arrange(-fStat)

## ----showHits, results='asis'-------------------------------------------------
knitr::kable(topHits)

## ----session------------------------------------------------------------------
devtools::session_info()

#' \code{NPARC} package
#'
#' Non-parametric analysis of response curves
#'
#' See the preprint on
#' \href{https://www.biorxiv.org/content/10.1101/373845v2}{Childs, Bach, Franken et al. (2019): Non-parametric analysis of thermal proteome profiles reveals novel drug-binding proteins}
#'
#' @docType package
#' @name NPARC
#' @importFrom rlang .data
#' @import dplyr
#' @import tidyr
#' @import BiocParallel
#' @importFrom magrittr %<>%
#' @importFrom stats coefficients fitted resid D vcov integrate predict nobs median mad nls na.omit nls.control runif
#' @importFrom broom augment
#' @importFrom MASS fitdistr
NULL

combineRSS <- function(idNull, rssNull, nCoeffsNull, nFittedNull,
                       idAlt, rssAlt, nCoeffsAlt, nFittedAlt){

  rssAggrNull <- aggregateRSS(id = idNull, rss = rssNull, nCoeffs = nCoeffsNull, nFitted = nFittedNull)
  rssAggrAlt  <- aggregateRSS(id = idAlt, rss = rssAlt,  nCoeffs = nCoeffsAlt,  nFitted = nFittedAlt)

  colnames(rssAggrNull)[-1] %<>% paste0(.data, "Null")
  colnames(rssAggrAlt)[-1] %<>% paste0(.data, "Alt")

  rss <- full_join(rssAggrNull, rssAggrAlt, by = "id") %>%
    mutate(rssDiff = rssNull - rssAlt)

  return(rss)
}

aggregateRSS <- function(id, rss, nCoeffs, nFitted){

  out <- tibble(id, rss, nCoeffs, nFitted) %>%
    group_by(id) %>%
    summarise(rss = sum(rss, na.rm = FALSE),
              nCoeffs = sum(nCoeffs, na.rm = FALSE),
              nFitted = sum(nFitted, na.rm = FALSE)) %>%
    ungroup

  return(out)
}


#' Perform F-test
#'
#' @param modelMetrics data.frame with results of the model fit in long format.
#' @param dfType character value indicating the method for degrees of freedom computation for the F-test. Theoretical yields the text-book solution. Empirical yields estimates derived from the distribution moments of the RSS.
#' @return data frame with fitted model parameters and additional columns listing e.g. residuals sum of squares of
#'  null and alterantive model and raw and adjusted p values retrieved from testing
#' @export
#' @importFrom stats pf p.adjust
#' @examples
#' data(stauro_TPP_data_tidy)
#' df <- dplyr::filter(stauro_TPP_data_tidy, grepl("CDK|GTP|CRK", uniqueID))
#' fits <- NPARCfit(x = df$temperature, 
#'                  y = df$relAbundance, 
#'                  id = df$uniqueID, 
#'                  groupsNull = NULL, 
#'                  groupsAlt = df$compoundConcentration, 
#'                  returnModels = FALSE)
#' modelMetrics <- fits$metrics
#' testRes <-  NPARCtest(modelMetrics, dfType = "theoretical")                     
NPARCtest <- function(modelMetrics, dfType = c("empirical", "theoretical")){

  metricsNull <- filter(modelMetrics, .data$modelType == "null")
  metricsAlt <- filter(modelMetrics, .data$modelType == "alternative")

  metrics <- combineRSS(idNull = metricsNull$id,
                        rssNull = metricsNull$rss,
                        nCoeffsNull = metricsNull$nCoeffs,
                        nFittedNull = metricsNull$nFitted,
                        idAlt = metricsAlt$id,
                        rssAlt = metricsAlt$rss,
                        nCoeffsAlt = metricsAlt$nCoeffs,
                        nFittedAlt = metricsAlt$nFitted)

  id <- metrics$id
  rss0 <- metrics$rssNull
  rss1 <- metrics$rssAlt
  pars0 <- metrics$nCoeffsNull
  pars1 <- metrics$nCoeffsAlt
  n0 <- metrics$nFittedNull
  n1 <- metrics$nFittedAlt
  rssDiff <- metrics$rssDiff

  if (dfType == "theoretical"){

    d1 = pars1 - pars0
    d2 = n1 - pars1

  } else if (dfType == "empirical"){

    distr_pars <- estimate_df(rss1 = rss1, rssDiff = rssDiff)
    d1 <- distr_pars$d1
    d2 <- distr_pars$d2
    s0_sq <- distr_pars$s0_sq
    rssDiff = rssDiff/s0_sq
    rss1 = rss1/s0_sq

  }

  f = (rssDiff/d1) / (rss1/d2)
  pVal = 1 - pf(f, df1 = d1, df2 = d2)
  pAdj = p.adjust(pVal, "BH")

  out <- tibble(id, rssDiff, fStat = f, pVal, pAdj, df1 = d1, df2 = d2, rssNull = rss0, rssAlt = rss1,
                nFittedNull = n0, nFittedAlt = n1, nCoeffsNull = pars0, nCoeffsAlt = pars1)

  return(out)
}

estimate_df <- function(rss1, rssDiff){!is.na(rssDiff)
  # Estimate degrees of freedom from distribution moments of the RSS

  rm_idx <- is.na(rssDiff) | (rssDiff <= 0)
  rss1 <- rss1[!rm_idx]
  rssDiff <- rssDiff[!rm_idx]

  M = median(rssDiff, na.rm = TRUE)
  V = mad(rssDiff, na.rm = TRUE)^2
  s0_sq = 1/2 * V/M
  rssDiff = rssDiff/s0_sq
  rss1 = rss1/s0_sq
  d1 = MASS::fitdistr(x = rssDiff, densfun = "chi-squared", start = list(df = 1), method = "Brent", lower = 0, upper = length(rssDiff))[["estimate"]]
  d2 = MASS::fitdistr(x = rss1, densfun = "chi-squared", start = list(df = 1),  method = "Brent", lower = 0, upper = length(rssDiff))[["estimate"]]

  out <- list(d1 = d1, d2 = d2, s0_sq = s0_sq)
  return(out)
}

source(file.path(rprojroot::find_package_root_file(), "tests/init_tests.R"))

test_that("nparcFTest_realdata", {

  metrics <- data.frame(modelType = c(rep("null", each = length(rss0_ref)), rep("alternative", each = length(rss1_ref))),
                        id = c(unique(atp_ids), unique(atp_ids)),
                        rss = c(rss0_ref, rss1_ref),
                        nFitted = c(n0_ref, n1_ref),
                        nCoeffs = c(rep(3, each = length(rss0_ref)), rep(6, each = length(rss1_ref))))

  testRes <- NPARCtest(metrics, dfType = "theoretical")

  # Compute expected values:
  d1 = 6 - 3
  d2 = n1_ref - 6
  f = ((rss0_ref-rss1_ref)/d1) / (rss1_ref/d2)
  p = 1 - pf(f, df1 = d1, df2 = d2)
  pAdj_exp = p.adjust(p, "BH")

  expect_equal(testRes$pAdj, pAdj_exp)

})

test_that("nparcFTest_single", {

  metrics <- data.frame(modelType = c("null", "alternative"),
                        id = c(1,1),
                        rss = c(2, 1),
                        nFitted = c(40, 30),
                        nCoeffs = c(3,6))

  # Compute expected values:
  d1 = 6 - 3
  d2 = 30 - 6
  f = ((2-1)/d1) / (1/d2)
  p = 1 - pf(f, df1 = d1, df2 = d2)
  pAdj_exp = p.adjust(p, "BH")

  pAdj <- NPARCtest(modelMetrics = metrics, dfType = "theoretical")$pAdj

  expect_equal(pAdj, pAdj_exp)
})

---
title: "Analysing thermal proteome profiling data with the NPARC package"
author: 
  - Dorothee Childs
  - Nils Kurzawa
package: NPARC
date: "`r format(Sys.time(), '%d %B %Y,   %X')`"
bibliography: bibliography.bib
output: 
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Analysing thermal proteome profiling data with the NPARC package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```


# Introduction
This vignette shows how to reproduce the analysis described by [Childs, Bach, Franken et al. (2019): Non-parametric analysis of thermal proteome profiles reveals novel drug-binding proteins](https://doi.org/10.1101/373845) using the `NPARC` package.

# Preparation

Load necessary packages.
```{r dependencies, message=FALSE}
library(dplyr)
library(magrittr)
library(ggplot2)
library(broom)
library(knitr)
library(NPARC)
```


# Data import

First, we load data from a staurosporine TPP experiment [@Savitski2014]. The necessary ETL (extract, transform, load) steps have already been conducted, including download from the supplements of the respective publication and conversion into tidy format. 

```{r load_data}
data("stauro_TPP_data_tidy")
```

Before applying any further transformations and filters we create a copy of the imported data.
```{r}
df <- stauro_TPP_data_tidy
```


Let's perform a first check of the imported data:
```{r data_head}
df %>% 
  mutate(compoundConcentration = factor(compoundConcentration), 
         replicate = factor(replicate), 
         dataset = factor(dataset)) %>% 
  summary()

```

The displayed data contains the following columns:

- `dataset`: The dataset containing the measurements of several TMT-10 experiments. In each experiment, cells were treated with a vehicle or with the compound in one or two concentrations, and measured at ten different temperatures.
- `uniqueID`: The unique identifier for each protein. In the given dataset, it contains the gene symbol concatenated by IPI id. Example: `r head(unique(df$uniqueID))` .
- `relAbundance`: The relative signal intensity of the protein in each experiment, scaled to the intensity at the lowest temperature.
- `temperature`: The temperatures corresponding to each of the ten measurements in a TMT experiment.
- `compoundConcentration` The concentration of the administered compound in $\mu M$.
- `replicate`: The replicate number in each experimental group. Each pair of vehicle and treatment experiments was conducted in two replicates.
- `uniquePeptideMatches`: The number of unique peptides with which a protein was identified.

# Data preprocessing and exploration

The imported data contains **`r nrow(df)`** rows with entries for **`r length(unique(df$uniqueID))`** proteins.

First, we remove all abundances that were not found with at least one unique peptide, or for which a missing value was recorded.
```{r filter_qupm_or_NAs, results='asis'}
df %<>% filter(uniquePeptideMatches >= 1)
df %<>% filter(!is.na(relAbundance))
```

Next, we ensure that the dataset only contains proteins reproducibly observed with full melting curves in both replicates and treatment groups per dataset.
A full melting curve is defined by the presence of measurements at all 10 temperatures for the given experimental group.

```{r count_curves_per_protein}
# Count full curves per protein
df %<>%
  group_by(dataset, uniqueID) %>%
  mutate(n = n()) %>%
  group_by(dataset) %>%
  mutate(max_n = max(n)) %>% 
  ungroup

table(distinct(df, uniqueID, n)$n)
```

We see that the majority of proteins contain 40 measurements. This corresponds to two full replicate curves per experimental group. We will focus on these in the current analysis.

```{r}
# Filter for full curves per protein:
df %<>% 
  filter(n == max_n) %>%
  dplyr::select(-n, -max_n)
```


The final data contains **`r nrow(df)`** rows with entries for **`r length(unique(df$uniqueID))`** proteins.
This number coincides with the value reported in Table 1 of the corresponding publication [@Childs2019].


# Illustrative example

We first illustrate the principles of nonparametric analysis of response curves (NPARC) on an example protein (STK4) from the staurosporine dataset. The same protein is shown in Figures 1 and 2 of the paper.

## Select data

We first select all entries belonging to the desired protein and dataset:
```{r select_stk4}
stk4 <- filter(df, uniqueID == "STK4_IPI00011488")
```

The table `stk4` has `r nrow(stk4)` rows with measurements of four experimental groups. They consist of two treatment groups (vehicle: $0~\mu M$ staurosporine, treatment: $20~\mu M$ staurosporine) with two replicates each. Let us look at the treatment group of replicate 1 for an example:
```{r}
stk4 %>% filter(compoundConcentration == 20, replicate == 1) %>% 
  dplyr::select(-dataset) %>% kable(digits = 2)
```


To obtain a first impression of the measurements in each experimental group, we generate a plot of the measurements:
```{r plot_stk4}
stk4_plot_orig <- ggplot(stk4, aes(x = temperature, y = relAbundance)) +
  geom_point(aes(shape = factor(replicate), color = factor(compoundConcentration)), size = 2) +
  theme_bw() +
  ggtitle("STK4") +
  scale_color_manual("staurosporine (mu M)", values = c("#808080", "#da7f2d")) +
  scale_shape_manual("replicate", values = c(19, 17))

print(stk4_plot_orig)
```

We will show how to add the fitted curves to this plot in the following steps.

## Define function for model fitting
To assess whether there is a significant difference between both treatment groups, we will fit a null model and an alternative models to the data. The null model fits a sigmoid melting curve through all data points irrespective of experimental condition. The alternative model fits separate melting curves per experimental group .

## Fit and plot null models
We use the `NPARC` package function `fitSingleSigmoid` to fit the null model:

```{r fit_null_stk4}
nullFit <- NPARC:::fitSingleSigmoid(x = stk4$temperature, y = stk4$relAbundance)
```

The function returns an object of class `nls`:
```{r summarize_null_stk4}
summary(nullFit)
```

The function `augment` from the `broom` package provides a convenient way to obtain the predictions and residuals at each temperature in tabular format. By appending the returned predictions and residuals to our measurements, we ensure that relevant data is collected in the same table and can be added to the plot for visualization. The residuals will be needed later for construction of the test statistic:

```{r augment_null_stk4}
nullPredictions <- broom::augment(nullFit)
```

Let us look at the values returned by `augment` at two consecutive temperatures. Note that, while the predictions will be the same for each experiment at a given temperature, the residuals will differ because they were computed by comparing the predictions to the actual measurements:
```{r head_augment_result}
nullPredictions %>% filter(x %in% c(46, 49)) %>% kable()
```


Now we can append these values to our data frame and show the predicted curve in the plot:
```{r add_null_resids_stk4}
stk4$nullPrediction <- nullPredictions$.fitted
stk4$nullResiduals <- nullPredictions$.resid

stk4_plot <- stk4_plot_orig + geom_line(data = stk4, aes(y = nullPrediction))

print(stk4_plot)
```

## Fit and plot alternative models

Next we fit the alternative model. Again, we compute the predicted values and the corresponding residuals by the `broom::augment()` function. To take the compound concentration as a factor into account, we iterate over both concentrations and fit separate models to each subset. We implement this by first grouping the data using the function `dplyr::group_by()`, and starting the model fitting by `dplyr::do()`.

```{r fit_alternative_stk4}
alternativePredictions <- stk4 %>%
# Fit separate curves per treatment group:
  group_by(compoundConcentration) %>%
  do({
    fit = NPARC:::fitSingleSigmoid(x = .$temperature, y = .$relAbundance, start=c(Pl = 0, a = 550, b = 10))
    broom::augment(fit)
  }) %>%
  ungroup %>%
  # Rename columns for merge to data frame:
  dplyr::rename(alternativePrediction = .fitted,
                alternativeResiduals = .resid,
                temperature = x,
                relAbundance = y)
```

Add the predicted values and corresponding residuals to our data frame:
```{r add_alternative_resids_stk4}
stk4 <- stk4 %>%
  left_join(alternativePredictions, 
            by = c("relAbundance", "temperature", 
                   "compoundConcentration")) %>%
  distinct()
```

Add the curves predicted by the alternative model to the plot. Conceptually, it corresponds to the plot shown in Figures 2 (A)/(B) of the paper.

```{r plot_Fig_2_A_B}
stk4_plot <- stk4_plot +
  geom_line(data = distinct(stk4, temperature, compoundConcentration, alternativePrediction), 
            aes(y = alternativePrediction, color = factor(compoundConcentration)))

print(stk4_plot)
```

This plot summarizes Figures 2(A) and 2(B) in the corresponding publication [@Childs2019].

## Compute RSS values

In order to quantify the improvement in goodness-of-fit of the alternative model relative to the null model, we compute the sum of squared residuals (RSS):

```{r compute_rss_stk4}
rssPerModel <- stk4 %>%
  summarise(rssNull = sum(nullResiduals^2),
            rssAlternative = sum(alternativeResiduals^2))

kable(rssPerModel, digits = 4)
```

These values will be used to construct the $F$-statistic according to

\begin{equation}
\label{eq:f_stat}
    {F} = \frac{{d}_{2}}{{d}_{1}} \cdot \frac{{RSS}^{0} - {RSS}^{1}}{{RSS}^{1}}.
\end{equation}

To compute this statistic and to derive a p-value, we need the degrees of freedom ${d}_{1}$ and ${d}_{2}$. As described in the paper, they cannot be analytically derived due to the correlated nature of the measurements. The paper describes how to estimate these values from the RSS-values of all proteins in the dataset. In the following Section, we illustrate how to repeat the model fitting for all proteins of a dataset and how to perform hypothesis testing on these models.

# Extend the analysis to all proteins 

This section describes the different steps of the NPARC workflow for model fitting and hyothesis testing. Note that the package also provides a function `runNPARC()` that performs all of the following steps with one single function call.


## Start fitting
In order to analyze all datasets as described in the paper, we fit null and alternative models to all proteins using the package function `NPARCfit`:

```{r}
BPPARAM <- BiocParallel::SerialParam(progressbar = FALSE)
```


```{r fit_all_proteins}
fits <- NPARCfit(x = df$temperature, 
                 y = df$relAbundance, 
                 id = df$uniqueID, 
                 groupsNull = NULL, 
                 groupsAlt = df$compoundConcentration, 
                 BPPARAM = BPPARAM,
                 returnModels = FALSE)

str(fits, 1)
```

The returned object `fits` contains two tables. The table `metrics` contains the fitted parameters and goodness-of-fit measures for the null and alternative models per protein and group. The table `predictions` contains the corresponding predicted values and residuals per model.

```{r show_fit_results}
fits$metrics %>% 
  mutate(modelType = factor(modelType), nCoeffs = factor(nCoeffs), nFitted = factor(nFitted), group = factor((group))) %>% 
  summary

fits$predictions %>% 
  mutate(modelType = factor(modelType), group = factor((group))) %>% 
  summary
```

## Check example

The results of the STK4 example from earlier can be selected from this object as follows.

First, we check the RSS values of the null and alterantive models:
```{r show_stk4_results}
stk4Metrics <- filter(fits$metrics, id == "STK4_IPI00011488")

rssNull <- filter(stk4Metrics, modelType == "null")$rss
rssAlt <- sum(filter(stk4Metrics, modelType == "alternative")$rss) # Summarize over both experimental groups

rssNull
rssAlt
```

Next, we plot the predicted curves per model and experimental group:
```{r plot_stk4_results}
stk4Predictions <- filter(fits$predictions, modelType == "alternative", id == "STK4_IPI00011488")

stk4_plot_orig +
  geom_line(data = filter(stk4Predictions, modelType == "alternative"), 
            aes(x = x, y = .fitted, color = factor(group))) +
    geom_line(data = filter(stk4Predictions, modelType == "null"), 
            aes(x = x, y = .fitted))
```

# Compute test statistics

## Why we need to estimate the degrees of freedom

In order to compute $F$-statistics per protein and dataset according to Equation (\ref{eq:f_stat}), we need to know the degrees of freedom of the corresponding null distribution. If we could assume independent and identically distributed (iid) residuals, we could compute them from the number of fitted values and model parameters. In the following, we will show why this simple equation is not appropriate for the curve data we are working with.

First, we compute the test statistics and p-values with theoretical degrees of freedom. These would be true for the case of iid residuals:
```{r testDOFiid}
modelMetrics <- fits$metrics 
fStats <- NPARCtest(modelMetrics, dfType = "theoretical")
```

Let us take a look at the computed degrees of freedom:
```{r plotF_DOFiid}
fStats %>% 
  filter(!is.na(pAdj)) %>%
  distinct(nFittedNull, nFittedAlt, nCoeffsNull, nCoeffsAlt, df1, df2) %>%
  kable()
```


We plot the $F$-statistics against the theoretical $F$-distribution to check how well the null distribution is approximated now:

```{r plotFiid}
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_density(aes(x = fStat), fill = "steelblue", alpha = 0.5) +
  geom_line(aes(x = fStat, y = df(fStat, df1 = df1, df2 = df2)), color = "darkred", size = 1.5) +
  theme_bw() +
  # Zoom in to small values to increase resolution for the proteins under H0:
  xlim(c(0, 10))
```

The densities of the theoretical $F$-distribution (red) do not fit the observed values (blue) very well. While the theoretical distribution tends to overestimate the number of proteins with test statistics smaller than 2.5, it appears to underestimate the amount of proteins with larger values.  This would imply that even for highly specific drugs, we observe many more significant differences than we would expect by chance. This hints at an anti-conservative behaviour of our test with the calculated degree of freedom parameters. This is reflected in the p-value distributions. If the distribution assumptions were met, we would expect the null cases to follow a uniform distribution, with a peak on the left for the non-null cases. Instead, we observe a tendency to obtain fewer values than expected in the middle range (around 0.5), but distinct peaks to the left.

```{r plotPiid}
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_histogram(aes(x = pVal, y = ..density..), fill = "steelblue", alpha = 0.5, boundary = 0, bins = 30) +
  geom_line(aes(x = pVal, y = dunif(pVal)), color = "darkred", size = 1.5) +
  theme_bw()
```

## How to estimate the degrees of freedom

In the paper, we describe an alternative way to estimate the degrees of freedom by fitting $\chi^2$ distributions to the numerator and denominator across all proteins in a dataset. To enable fitting of the distributions, we first need to re-scale the variables by a scaling factor. Because the scaling factors are characteristic for each dataset (it depends on the variances of the residuals in the respective dataset), we estimate them from the data according to:

\begin{align} \label{eq:scale-param}
\sigma_0^2 &= \frac{1}{2} \frac{V}{M},
\end{align}

where $V$ is the variance of the distribution, and $M$ is the mean of the distribution.

The following functin estimates $V$ and $M$ from the empirical distributions of the RSS differences $({RSS}^1 - {RSS}^0)$. To increase robustness, it estimate $M$ and $V$ by their D-estimates @Marazzi2002 (median and median absolute deviation). 
It then scales the numerator and denominator of the $F$-statistic by these scaling factors and estimate the degree of freedom parameters by fitting unscaled $\chi^2$ distributions. Finally, it fits the degrees of freedom parameters numerically, computes the test statistics according to Equation (\ref{eq:f_stat}) and derives p-values.


```{r testDOFemp}
modelMetrics <- fits$metrics 
fStats <- NPARCtest(modelMetrics, dfType = "empirical")
```



We plot the $F$-statistics against the theoretical $F$-distribution to check how well the null distribution is approximated now:

```{r plotFnew}
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_density(aes(x = fStat), fill = "steelblue", alpha = 0.5) +
  geom_line(aes(x = fStat, y = df(fStat, df1 = df1, df2 = df2)), color = "darkred", size = 1.5) +
  theme_bw() +
  # Zoom in to small values to increase resolution for the proteins under H0:
  xlim(c(0, 10))
```

Also check the p-value histograms. We expect the null cases to follow a uniform distribution, with a peak on the left for the non-null cases:

```{r plotPnew}
ggplot(filter(fStats, !is.na(pAdj))) +
  geom_histogram(aes(x = pVal, y = ..density..), fill = "steelblue", alpha = 0.5, boundary = 0, bins = 30) +
  geom_line(aes(x = pVal, y = dunif(pVal)), color = "darkred", size = 1.5) +
  theme_bw()
```

The $F$-statistics and p-values approximate the expected distributions substantially closer when based on the estimated degrees of freedom than when based on the theoretical degrees of freedom.

# Detect significantly shifted proteins

Finally, we can select proteins that are significantly shifted by putting a threshold on the Benjamini-Hochberg corrected p-values.

```{r selectHits}
topHits <- fStats %>% 
  filter(pAdj <= 0.01) %>%
  dplyr::select(id, fStat, pVal, pAdj) %>%
  arrange(-fStat)
```

The table `topHits` contains **`r nrow(topHits)`** proteins with Benjamini-Hochberg corrected p-values $\leq 0.01$.


Let us look at the targets detected in each dataset. The same proteins are shown in Fig. S3, S4, S6, and S7 of the paper.
```{r showHits, results='asis'}
knitr::kable(topHits)
```


# Session info

```{r session}
devtools::session_info()
```

# Bibliography

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_fitting.R
\name{NPARCfit}
\alias{NPARCfit}
\title{Fit null and alternative models for Non-parametric analysis of response curves}
\usage{
NPARCfit(x, y, id, control = getParams(), groupsNull = NULL, groupsAlt,
  BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
  returnModels = FALSE)
}
\arguments{
\item{x}{numeric vector of the independent variables (typically temperature)}

\item{y}{numeric vector of the dependent variables (typically relative abundance measurements)}

\item{id}{character vector with the protein ID to which each each data point belongs.}

\item{control}{list of parameters used to control specific parts of the analyse}

\item{groupsNull}{one or more vectors with grouping variables for the null models. See details.}

\item{groupsAlt}{one or more vectors with grouping variables for the alternative models. See details.}

\item{BPPARAM}{BiocParallel parameter object to invoke curve fitting in parallel. Default: BiocParallel::SerialParam()}

\item{returnModels}{boolean value. If true, the fitted models are returned together with the test results}
}
\value{
data frame with fitted model parameters and additional columns listing e.g. residuals sum of squares
}
\description{
Fit melting curve and return model metrics as well as predictions for the null and alternative models.
}
\details{
\code{groupsNull} or \code{groupsAlt} can either be a single vector each, or data.frames of the same length as \code{x} and \code{y} with one column per factor
}
\examples{
data(stauro_TPP_data_tidy)
df <- dplyr::filter(stauro_TPP_data_tidy, grepl("CDK|GTP|CRK", uniqueID))
testResults <- NPARCfit(x = df$temperature,
                     y = df$relAbundance,
                     id = df$uniqueID,
                     groupsAlt = df$compoundConcentration)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stauro_TPP_data_tidy.R
\docType{data}
\name{stauro_TPP_data_tidy}
\alias{stauro_TPP_data_tidy}
\title{TPP dataset of staurosporine treated cells.}
\format{An object of class \code{"data.frame"}}
\usage{
data(stauro_TPP_data_tidy)
}
\description{
Data from a thermal proteome profiling (TPP) experiment investigating the
ATP-competitive pan-kinase inhibitor staurosporine on K562 cells.
The data has been downloaded the data from the supplement of the respective publication and converted into tidy format.
}
\references{
Savitski et al. (2014): Tracking cancer drugs in living cells by thermal profiling of the proteome. Science 346, 1255784.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_fitting.R
\name{fitSingleSigmoid}
\alias{fitSingleSigmoid}
\title{Fit sigmoid model}
\usage{
fitSingleSigmoid(x, y, start = c(Pl = 0, a = 550, b = 10))
}
\arguments{
\item{x}{numeric vector of the independent variables (typically temperature)}

\item{y}{numeric vector of the dependent variables (typically relative abundance measurements)}

\item{start}{numeric vector of start parameters for the melting curve equation}
}
\value{
model summary of type "nls"
}
\description{
Fit sigmoid model
}
\details{
Fits the following function to the data:
\eqn{y = (1 - Pl)  / (1+exp((b - a/x))) + Pl}
}
\examples{
data(stauro_TPP_data_tidy)
stk4 <- dplyr::filter(stauro_TPP_data_tidy, grepl("STK4", uniqueID))
fitSingleSigmoid(stk4$temperature, stk4$relAbundance)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testing.R
\name{NPARCtest}
\alias{NPARCtest}
\title{Perform F-test}
\usage{
NPARCtest(modelMetrics, dfType = c("empirical", "theoretical"))
}
\arguments{
\item{modelMetrics}{data.frame with results of the model fit in long format.}

\item{dfType}{character value indicating the method for degrees of freedom computation for the F-test. Theoretical yields the text-book solution. Empirical yields estimates derived from the distribution moments of the RSS.}
}
\value{
data frame with fitted model parameters and additional columns listing e.g. residuals sum of squares of
 null and alterantive model and raw and adjusted p values retrieved from testing
}
\description{
Perform F-test
}
\examples{
data(stauro_TPP_data_tidy)
df <- dplyr::filter(stauro_TPP_data_tidy, grepl("CDK|GTP|CRK", uniqueID))
fits <- NPARCfit(x = df$temperature, 
                 y = df$relAbundance, 
                 id = df$uniqueID, 
                 groupsNull = NULL, 
                 groupsAlt = df$compoundConcentration, 
                 returnModels = FALSE)
modelMetrics <- fits$metrics
testRes <-  NPARCtest(modelMetrics, dfType = "theoretical")                     
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NPARC.R
\docType{package}
\name{NPARC}
\alias{NPARC}
\alias{NPARC-package}
\title{\code{NPARC} package}
\description{
Non-parametric analysis of response curves
}
\details{
See the preprint on
\href{https://www.biorxiv.org/content/10.1101/373845v2}{Childs, Bach, Franken et al. (2019): Non-parametric analysis of thermal proteome profiles reveals novel drug-binding proteins}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_fitting.R
\name{getParams}
\alias{getParams}
\title{Control parameters for model fitting}
\usage{
getParams(start = c(Pl = 0, a = 550, b = 10), maxAttempts = 100)
}
\arguments{
\item{start}{Numeric vector of start parameters for the melting curve equation}

\item{maxAttempts}{Number of resampling steps in case of unsuccessful model fits}
}
\value{
list of two elements: 1) "start" listing the starting parameters for
 melting curve fitting, 2) "maxAttempts" listing the maximal number of attempts
 the fit should be allowed
}
\description{
Control parameters for model fitting
}
\examples{
data(stauro_TPP_data_tidy)
df <- dplyr::filter(stauro_TPP_data_tidy, grepl("MAPK|ATP|CDK|GTP|CRK", uniqueID))
testResults <- runNPARC(x = df$temperature,
                     y = df$relAbundance,
                     id = df$uniqueID,
                     groupsAlt = df$compoundConcentration,
                     dfType = "empirical",
                     control = getParams(maxAttempts = 50))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{runNPARC}
\alias{runNPARC}
\title{Non-parametric analysis of response curves}
\usage{
runNPARC(x, y, id, groupsNull = NULL, groupsAlt,
  BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
  dfType = c("theoretical", "empirical"), control = getParams())
}
\arguments{
\item{x}{numeric vector of the independent variables (typically temperature)}

\item{y}{numeric vector of the dependent variables (typically relative abundance measurements)}

\item{id}{character vector with the protein ID to which each each data point belongs.}

\item{groupsNull}{one or more vectors with grouping variables for the null models. See details.}

\item{groupsAlt}{one or more vectors with grouping variables for the alternative models. See details.}

\item{BPPARAM}{BiocParallel parameter object to invoke curve fitting in parallel. Default: BiocParallel::SerialParam()}

\item{dfType}{character value indicating the method for degrees of freedom computation for the F-test. 
Theoretical yields the text-book solution. Empirical yields estimates derived from the distribution moments of the RSS.}

\item{control}{list of parameters used to control specific parts of the analyse}
}
\value{
data frame with fitted model parameters and additional columns listing e.g. residuals sum of squares of
 null and alterantive model
}
\description{
Wrapper function for melting curve fitting and hypothesis testing.
}
\details{
\code{groupsNull} or \code{groupsAlt} can either be a single vector each, or data.frames of the same length as \code{x} and \code{y} with one column per factor
}
\examples{
data(stauro_TPP_data_tidy)
df <- dplyr::filter(stauro_TPP_data_tidy, grepl("CDK|GTP|CRK", uniqueID))
testResults <- runNPARC(x = df$temperature,
                     y = df$relAbundance,
                     id = df$uniqueID,
                     groupsAlt = df$compoundConcentration,
                     dfType = "empirical")
}
